@@ -1,4 +1,5 @@
 from .agent import Agent
+from .router import AgentRouter
 from .types import AIConfig, MemoryConfig, ReasonerDefinition, SkillDefinition
 from .multimodal import (
     Text, Image, Audio, File, MultimodalContent,
@@ -16,6 +17,7 @@ __all__ = [
     "MemoryConfig",
     "ReasonerDefinition",
     "SkillDefinition",
+    "AgentRouter",
     # Input multimodal classes
     "Text",
     "Image",
