@@ -5,10 +5,26 @@ from typing import Any, Dict, List, Literal, Optional, Type, Union
 
 import requests
 from brain_sdk.agent_utils import AgentUtils
-from brain_sdk.logger import log_debug
+from brain_sdk.logger import log_debug, log_error, log_warn
 from brain_sdk.rate_limiter import StatelessRateLimiter
 from httpx import HTTPStatusError
 from pydantic import BaseModel
+# Expose module-level symbols for patching in tests
+try:
+    import litellm as litellm  # type: ignore
+except Exception:  # pragma: no cover - test environments may not have litellm
+    class _LiteLLMStub:
+        pass
+    litellm = _LiteLLMStub()  # type: ignore
+
+try:
+    import openai as openai  # type: ignore
+except Exception:  # pragma: no cover - test environments may not have openai
+    class _OpenAIStub:
+        class OpenAI:
+            pass
+    openai = _OpenAIStub()  # type: ignore
+
 
 
 class AgentAI:
@@ -66,7 +82,6 @@ class AgentAI:
                     )
 
                 self._initialization_complete = True
-                log_debug("Model limits cached successfully")
 
             except Exception as e:
                 log_debug(f"Failed to cache model limits: {e}")
@@ -233,7 +248,6 @@ class AgentAI:
             if processed_content:
                 messages.extend(processed_content)
 
-        # Integrate LiteLLM call here
         try:
             import litellm
         except ImportError:
@@ -260,22 +274,6 @@ class AgentAI:
         if hasattr(final_config, "max_input_tokens") and final_config.max_input_tokens:
             model_context_length = final_config.max_input_tokens
             detection_method = "explicit_config"
-            log_debug(
-                f"Using explicit max_input_tokens from config: {model_context_length}"
-            )
-
-        # Strategy 2: Try LiteLLM's get_max_tokens
-        if not model_context_length:
-            try:
-                model_context_length = get_max_tokens(final_config.model)
-                detection_method = "litellm_detection"
-                log_debug(
-                    f"LiteLLM detected context length for {final_config.model}: {model_context_length}"
-                )
-            except Exception as e:
-                log_debug(
-                    f"LiteLLM get_max_tokens failed for model {final_config.model}: {e}"
-                )
 
         # Strategy 3: Use fallback model mappings
         if not model_context_length:
@@ -285,17 +283,11 @@ class AgentAI:
                 )
                 if model_context_length:
                     detection_method = "fallback_mapping"
-                    log_debug(
-                        f"Using fallback mapping for {final_config.model}: {model_context_length}"
-                    )
 
         # Strategy 4: Conservative fallback with warning
         if not model_context_length:
             model_context_length = 10192  # More reasonable than 4096
             detection_method = "conservative_fallback"
-            log_debug(
-                f"WARNING: Could not detect context length for {final_config.model}, using conservative fallback: {model_context_length}"
-            )
 
         # Calculate safe input token limit: context_length - max_output_tokens - buffer
         output_tokens = (
@@ -307,15 +299,8 @@ class AgentAI:
             1000, model_context_length - output_tokens - buffer_tokens
         )
 
-        log_debug(
-            f"Token calculation for {final_config.model}: context={model_context_length}, output={output_tokens}, buffer={buffer_tokens}, safe_input={safe_input_limit} (method: {detection_method})"
-        )
-
         # Validate the calculation makes sense
         if safe_input_limit < 1000:
-            log_debug(
-                f"WARNING: Calculated safe_input_limit ({safe_input_limit}) is very low. Using minimum of 1000 tokens."
-            )
             safe_input_limit = 1000
 
         # Count actual prompt tokens using LiteLLM's token counter
@@ -323,9 +308,6 @@ class AgentAI:
             actual_prompt_tokens = token_counter(
                 model=final_config.model, messages=messages
             )
-            log_debug(
-                f"Actual prompt tokens: {actual_prompt_tokens}, safe limit: {safe_input_limit}"
-            )
         except Exception as e:
             log_debug(f"Could not count prompt tokens, proceeding with trimming: {e}")
             actual_prompt_tokens = (
@@ -334,21 +316,15 @@ class AgentAI:
 
         # Only trim if necessary based on actual token count
         if actual_prompt_tokens > safe_input_limit:
-            log_debug(
-                f"Prompt ({actual_prompt_tokens} tokens) exceeds safe limit ({safe_input_limit}), applying intelligent trimming"
-            )
             trimmed_messages = trim_messages(
                 messages, final_config.model, max_tokens=safe_input_limit
             )
             if len(trimmed_messages) != len(messages) or any(
                 m1 != m2 for m1, m2 in zip(messages, trimmed_messages)
             ):
-                log_debug(f"Applied token-aware trimming via LiteLLM")
                 messages = trimmed_messages
         else:
-            log_debug(
-                f"Prompt ({actual_prompt_tokens} tokens) is within safe limit ({safe_input_limit}), no trimming needed"
-            )
+            pass
 
         # Prepare LiteLLM parameters using the config's method
         # This leverages LiteLLM's standard environment variable handling and smart token management
@@ -363,14 +339,7 @@ class AgentAI:
             # Use LiteLLM's native Pydantic model support for structured outputs
             litellm_params["response_format"] = schema
 
-        # Log LiteLLM call with truncated parameters for debugging
-        log_debug(
-            "Making LiteLLM call",
-            payload={
-                k: str(v)[:100] + "..." if len(str(v)) > 100 else v
-                for k, v in litellm_params.items()
-            },
-        )
+        litellm_params["transforms"] = ["middle-out"]
 
         # Define the LiteLLM call function for rate limiter
         async def _make_litellm_call():
@@ -464,8 +433,8 @@ class AgentAI:
                     json_data = json.loads(str(multimodal_response.text))
                     return schema(**json_data)
                 except (json.JSONDecodeError, ValueError) as parse_error:
-                    print(f"Failed to parse JSON response: {parse_error}")
-                    print(f"Raw response: {multimodal_response.text}")
+                    log_error(f"Failed to parse JSON response: {parse_error}")
+                    log_debug(f"Raw response: {multimodal_response.text}")
                     # Fallback: try to extract JSON from the response
                     json_match = re.search(
                         r"\{.*\}", str(multimodal_response.text), re.DOTALL
@@ -485,7 +454,7 @@ class AgentAI:
 
     def _process_multimodal_args(self, args: tuple) -> List[Dict[str, Any]]:
         """Process multimodal arguments into LiteLLM-compatible message format"""
-        from .multimodal import Audio, File, Image, Text
+        from brain_sdk.multimodal import Audio, File, Image, Text
 
         messages = []
         user_content = []
@@ -559,7 +528,7 @@ class AgentAI:
                             }
                         )
                     except Exception as e:
-                        print(f"Warning: Could not read image file {arg}: {e}")
+                        log_warn(f"Could not read image file {arg}: {e}")
                         user_content.append(
                             {"type": "text", "text": f"[Image file: {arg}]"}
                         )
@@ -588,7 +557,7 @@ class AgentAI:
                             }
                         )
                     except Exception as e:
-                        print(f"Warning: Could not read audio file {arg}: {e}")
+                        log_warn(f"Could not read audio file {arg}: {e}")
                         user_content.append(
                             {
                                 "type": "text",
@@ -635,7 +604,7 @@ class AgentAI:
                                 {"type": "text", "text": "[Audio data provided]"}
                             )
                     except Exception as e:
-                        print(f"Warning: Could not process audio base64: {e}")
+                        log_warn(f"Could not process audio base64: {e}")
                         user_content.append(
                             {"type": "text", "text": "[Audio data provided]"}
                         )
@@ -664,7 +633,7 @@ class AgentAI:
                             }
                         )
                     except Exception as e:
-                        print(f"Warning: Could not process image bytes: {e}")
+                        log_warn(f"Could not process image bytes: {e}")
                         user_content.append(
                             {"type": "text", "text": "[Image data provided]"}
                         )
@@ -693,7 +662,7 @@ class AgentAI:
                             }
                         )
                     except Exception as e:
-                        print(f"Warning: Could not process audio bytes: {e}")
+                        log_warn(f"Could not process audio bytes: {e}")
                         user_content.append(
                             {"type": "text", "text": "[Audio data provided]"}
                         )
@@ -902,7 +871,7 @@ class AgentAI:
 
                 audio_data = base64.b64encode(binary_content).decode("utf-8")
             except Exception as e:
-                print(f"Failed to process audio response: {e}")
+                log_error(f"Failed to process audio response: {e}")
                 # Use a placeholder for now
                 audio_data = ""
 
@@ -920,7 +889,7 @@ class AgentAI:
 
         except Exception as e:
             # Fallback to text-only MultimodalResponse
-            print(f"TTS generation failed: {e}")
+            log_error(f"TTS generation failed: {e}")
             return MultimodalResponse(
                 text=text_input,
                 audio=None,
@@ -1030,7 +999,7 @@ class AgentAI:
 
         except Exception as e:
             # Fallback to text-only MultimodalResponse
-            print(f"OpenAI direct audio generation failed: {e}")
+            log_error(f"OpenAI direct audio generation failed: {e}")
             return MultimodalResponse(
                 text=text_input,
                 audio=None,
@@ -1095,13 +1064,13 @@ class AgentAI:
             response = await litellm.aimage_generation(**image_params)
 
             # Import multimodal response detection
-            from .multimodal_response import detect_multimodal_response
+            from brain_sdk.multimodal_response import detect_multimodal_response
 
             # Detect and wrap multimodal content
             return detect_multimodal_response(response)
 
         except Exception as e:
-            print(f"Image generation failed: {e}")
+            log_error(f"Image generation failed: {e}")
             raise
 
     async def ai_with_multimodal(
