@@ -7,6 +7,7 @@ from typing import Any, Dict, List, Optional
 from brain_sdk.agent_utils import AgentUtils
 from brain_sdk.dynamic_skills import DynamicMCPSkillManager
 from brain_sdk.execution_context import ExecutionContext
+from brain_sdk.logger import log_debug, log_error, log_info, log_warn
 from brain_sdk.mcp_client import MCPClientRegistry
 from brain_sdk.mcp_manager import MCPManager
 from brain_sdk.types import AgentStatus, MCPServerHealth
@@ -49,7 +50,7 @@ class AgentMCP:
         for item in current_dir.iterdir():
             if item.is_dir() and (item / "packages" / "mcp").exists():
                 if self.agent.dev_mode:
-                    print(f"🔍 Found agent directory: {item}")
+                    log_debug(f"Found agent directory: {item}")
                 return str(item)
 
         # Look in parent directories (up to 3 levels)
@@ -57,13 +58,13 @@ class AgentMCP:
             parent = current_dir.parents[i] if i < len(current_dir.parents) else None
             if parent and (parent / "packages" / "mcp").exists():
                 if self.agent.dev_mode:
-                    print(f"🔍 Found agent directory in parent: {parent}")
+                    log_debug(f"Found agent directory in parent: {parent}")
                 return str(parent)
 
         # Fallback to current directory
         if self.agent.dev_mode:
-            print(
-                f"⚠️ No packages/mcp directory found, using current directory: {current_dir}"
+            log_warn(
+                f"No packages/mcp directory found, using current directory: {current_dir}"
             )
         return str(current_dir)
 
@@ -80,17 +81,17 @@ class AgentMCP:
             self.agent.mcp_client_registry = MCPClientRegistry(self.agent.dev_mode)
 
             if self.agent.dev_mode:
-                print(f"🔌 Initialized MCP Manager in {agent_dir}")
+                log_info(f"Initialized MCP Manager in {agent_dir}")
 
             # Initialize Dynamic Skill Manager when both MCP components are available
             if self.agent.mcp_manager and self.agent.mcp_client_registry:
                 self.agent.dynamic_skill_manager = DynamicMCPSkillManager(self.agent, self.agent.dev_mode)
                 if self.agent.dev_mode:
-                    print("🔧 Dynamic MCP skill manager initialized")
+                    log_info("Dynamic MCP skill manager initialized")
 
         except Exception as e:
             if self.agent.dev_mode:
-                print(f"⚠️ Failed to initialize MCP Manager: {e}")
+                log_error(f"Failed to initialize MCP Manager: {e}")
             self.agent.mcp_manager = None
             self.agent.mcp_client_registry = None
             self.agent.dynamic_skill_manager = None
@@ -99,12 +100,12 @@ class AgentMCP:
         """Start all configured MCP servers using SimpleMCPManager."""
         if not self.agent.mcp_manager:
             if self.agent.dev_mode:
-                print("💡 No MCP Manager available - skipping server startup")
+                log_info("No MCP Manager available - skipping server startup")
             return
 
         try:
             if self.agent.dev_mode:
-                print("🔌 Starting MCP servers...")
+                log_info("Starting MCP servers...")
 
             # Start all servers
             started_servers = await self.agent.mcp_manager.start_all_servers()
@@ -112,13 +113,13 @@ class AgentMCP:
             if started_servers:
                 successful = sum(1 for success in started_servers.values() if success)
                 if self.agent.dev_mode:
-                    print(f"✅ Started {successful}/{len(started_servers)} MCP servers")
+                    log_info(f"Started {successful}/{len(started_servers)} MCP servers")
             elif self.agent.dev_mode:
-                print("💡 No MCP servers configured to start")
+                log_info("No MCP servers configured to start")
 
         except Exception as e:
             if self.agent.dev_mode:
-                print(f"⚠️ MCP server startup error: {e}")
+                log_error(f"MCP server startup error: {e}")
 
     def _cleanup_mcp_servers(self) -> None:
         """
@@ -129,18 +130,18 @@ class AgentMCP:
         """
         if not self.agent.mcp_manager:
             if self.agent.dev_mode:
-                print("💡 No MCP Manager available - skipping cleanup")
+                log_info("No MCP Manager available - skipping cleanup")
             return
 
         async def async_cleanup():
             try:
                 if self.agent.dev_mode:
-                    print("🔌 Stopping MCP servers...")
+                    log_info("Stopping MCP servers...")
 
                 # Check if mcp_manager is still available
                 if not self.agent.mcp_manager:
                     if self.agent.dev_mode:
-                        print("💡 MCP Manager not available during cleanup")
+                        log_info("MCP Manager not available during cleanup")
                     return
 
                 # Get current server status before stopping
@@ -162,18 +163,18 @@ class AgentMCP:
                                     if self.agent.dev_mode:
                                         health = all_status.get(alias, {})
                                         pid = health.get("pid") or "N/A"
-                                        print(f"✅ Stopped MCP server: {alias} (PID: {pid})")
+                                        log_info(f"Stopped MCP server: {alias} (PID: {pid})")
                             except Exception as e:
                                 if self.agent.dev_mode:
-                                    print(f"⚠️ Failed to stop MCP server {alias}: {e}")
+                                    log_error(f"Failed to stop MCP server {alias}: {e}")
 
                         if self.agent.dev_mode:
-                            print(f"🔌 Stopped {len(running_servers)} MCP servers")
+                            log_info(f"Stopped {len(running_servers)} MCP servers")
                     elif self.agent.dev_mode:
-                        print("💡 No running MCP servers to stop")
+                        log_info("No running MCP servers to stop")
             except Exception as e:
                 if self.agent.dev_mode:
-                    print(f"⚠️ Error during MCP server cleanup: {e}")
+                    log_error(f"Error during MCP server cleanup: {e}")
                 # Continue with shutdown even if cleanup fails
 
         # Run the async cleanup properly
@@ -187,7 +188,7 @@ class AgentMCP:
                 def handle_task_completion(t):
                     try:
                         if t.exception() is not None and self.agent.dev_mode:
-                            print(f"⚠️ MCP cleanup task failed: {t.exception()}")
+                            log_error(f"MCP cleanup task failed: {t.exception()}")
                     except Exception:
                         # Suppress any callback exceptions to prevent warnings
                         pass
@@ -202,10 +203,10 @@ class AgentMCP:
                     asyncio.run(async_cleanup())
                 except Exception as cleanup_error:
                     if self.agent.dev_mode:
-                        print(f"⚠️ MCP cleanup failed: {cleanup_error}")
+                        log_error(f"MCP cleanup failed: {cleanup_error}")
         except Exception as e:
             if self.agent.dev_mode:
-                print(f"⚠️ Failed to run MCP cleanup: {e}")
+                log_error(f"Failed to run MCP cleanup: {e}")
 
     def _register_mcp_server_skills(self) -> None:
         """
@@ -213,7 +214,7 @@ class AgentMCP:
         The static file-based approach is broken after SimpleMCPManager refactor.
         """
         if self.agent.dev_mode:
-            print("⚠️ DEPRECATED: _register_mcp_server_skills() is no longer used")
+            log_warn("DEPRECATED: _register_mcp_server_skills() is no longer used")
         return
 
     def _register_mcp_tool_as_skill(
@@ -229,7 +230,7 @@ class AgentMCP:
         tool_name = tool.get("name", "")
         if not tool_name:
             if self.agent.dev_mode:
-                print(f"⚠️ Skipping tool with missing name: {tool}")
+                log_warn(f"Skipping tool with missing name: {tool}")
             return
 
         skill_name = f"{server_alias}_{tool_name}"
@@ -255,8 +256,8 @@ class AgentMCP:
         async def mcp_skill_function(**kwargs):
             """Dynamically created MCP skill function"""
             if self.agent.dev_mode:
-                print(
-                    f"🔧 MCP skill called: {server_alias}.{tool_name} with args: {kwargs}"
+                log_debug(
+                    f"MCP skill called: {server_alias}.{tool_name} with args: {kwargs}"
                 )
 
             try:
@@ -279,7 +280,7 @@ class AgentMCP:
 
             except Exception as e:
                 if self.agent.dev_mode:
-                    print(f"❌ MCP skill error: {e}")
+                    log_error(f"MCP skill error: {e}")
                 return {
                     "status": "error",
                     "error": str(e),
@@ -311,7 +312,7 @@ class AgentMCP:
         self.agent.skills.append(
             {
                 "id": skill_name,
-                "input_schema": InputSchema.schema(),
+                "input_schema": InputSchema.model_json_schema(),
                 "tags": ["mcp", server_alias],
                 "description": tool.get("description", f"MCP tool: {tool_name}"),
             }
@@ -406,7 +407,7 @@ class AgentMCP:
         self.agent.skills.append(
             {
                 "id": skill_name,
-                "input_schema": input_schema.schema(),
+                "input_schema": input_schema.model_json_schema(),
                 "tags": ["mcp", server_alias],
             }
         )
@@ -453,7 +454,7 @@ class AgentMCP:
 
             except Exception as e:
                 if self.agent.dev_mode:
-                    print(f"⚠️ Error getting MCP server health: {e}")
+                    log_error(f"Error getting MCP server health: {e}")
 
         return mcp_servers
 
@@ -463,7 +464,7 @@ class AgentMCP:
         """
         try:
             if self.agent.dev_mode:
-                print("🔄 Background MCP initialization started")
+                log_info("Background MCP initialization started")
 
             # Start MCP servers
             if self.agent.mcp_manager:
@@ -482,8 +483,8 @@ class AgentMCP:
                 total = len(results)
 
                 if self.agent.dev_mode:
-                    print(
-                        f"✅ MCP initialization: {successful}/{total} servers started"
+                    log_info(
+                        f"MCP initialization: {successful}/{total} servers started"
                     )
 
                 # Update status based on MCP results
@@ -499,12 +500,12 @@ class AgentMCP:
                 # No MCP manager, agent is ready
                 self.agent._current_status = AgentStatus.READY
                 if self.agent.dev_mode:
-                    print("✅ No MCP servers to initialize - agent ready")
+                    log_info("No MCP servers to initialize - agent ready")
 
             # Register dynamic skills if available
             if self.agent.dynamic_skill_manager:
                 if self.agent.dev_mode:
-                    print("🔧 Registering MCP tools as skills...")
+                    log_info("Registering MCP tools as skills...")
                 await self.agent.dynamic_skill_manager.discover_and_register_all_skills()
 
             self.agent._mcp_initialization_complete = True
@@ -513,12 +514,12 @@ class AgentMCP:
             await self.agent.brain_handler.send_enhanced_heartbeat()
 
             if self.agent.dev_mode:
-                print(
-                    f"🎉 Background initialization complete - Status: {self.agent._current_status.value}"
+                log_info(
+                    f"Background initialization complete - Status: {self.agent._current_status.value}"
                 )
 
         except Exception as e:
             if self.agent.dev_mode:
-                print(f"⚠️ Background MCP initialization error: {e}")
+                log_error(f"Background MCP initialization error: {e}")
             self.agent._current_status = AgentStatus.DEGRADED
-            await self.agent.brain_handler.send_enhanced_heartbeat()
\ No newline at end of file
+            await self.agent.brain_handler.send_enhanced_heartbeat()
