@@ -13,12 +13,6 @@ from brain_sdk.logger import log_track, log_fire, log_debug, log_warn, log_error
 from brain_sdk.pydantic_utils import convert_function_args, should_convert_args
 from pydantic import ValidationError
 
-# Import aiohttp for fire-and-forget HTTP calls
-try:
-    import aiohttp
-except ImportError:
-    aiohttp = None
-
 
 class AgentWorkflow:
     """
@@ -62,10 +56,10 @@ class AgentWorkflow:
                 if func_name in frame.f_globals:
                     frame.f_globals[func_name] = tracked_func
                     if self.agent.dev_mode:
-                        print(f"🔍 SETUP: Replaced {func_name} in caller's globals")
+                        log_debug(f"SETUP: Replaced {func_name} in caller's globals")
             except Exception as e:
                 if self.agent.dev_mode:
-                    print(f"🔍 SETUP: Could not replace in caller's globals: {e}")
+                    log_warn(f"SETUP: Could not replace {func_name} in caller's globals: {e}")
 
             # 3. Replace in original function's module
             try:
@@ -74,12 +68,14 @@ class AgentWorkflow:
                     if module and hasattr(module, func_name):
                         setattr(module, func_name, tracked_func)
                         if self.agent.dev_mode:
-                            print(
-                                f"🔍 SETUP: Replaced {func_name} in module {original_func.__module__}"
+                            log_debug(
+                                f"SETUP: Replaced {func_name} in module {original_func.__module__}"
                             )
             except Exception as e:
                 if self.agent.dev_mode:
-                    print(f"🔍 SETUP: Could not replace in module: {e}")
+                    log_warn(
+                        f"SETUP: Could not replace {func_name} in module {original_func.__module__}: {e}"
+                    )
 
             # 4. Store reference to original function for debugging
             try:
@@ -89,11 +85,87 @@ class AgentWorkflow:
                 pass
 
             if self.agent.dev_mode:
-                print(f"🔍 SETUP: Function replacement complete for {func_name}")
+                log_debug(f"SETUP: Function replacement complete for {func_name}")
 
         except Exception as e:
             if self.agent.dev_mode:
-                print(f"⚠️ Function replacement failed for {func_name}: {e}")
+                log_error(f"Function replacement failed for {func_name}: {e}")
+
+    async def _ensure_execution_registered(
+        self,
+        context: ExecutionContext,
+        reasoner_name: str,
+        parent_context: Optional[ExecutionContext],
+    ) -> None:
+        """Ensure the given execution context is registered with Brain."""
+
+        if getattr(context, "registered", False):
+            return
+
+        client = getattr(self.agent, "client", None)
+        async_request = getattr(client, "_async_request", None)
+
+        run_id = context.run_id or (parent_context.run_id if parent_context else None)
+        if run_id is None:
+            # Fallback so registration still succeeds and the server assigns one.
+            run_id = context.workflow_id
+            context.run_id = run_id
+
+        if async_request is None or not callable(async_request):
+            # Nothing we can do; mark as registered to avoid repeated attempts.
+            context.registered = True
+            return
+
+        payload = {
+            "run_id": run_id,
+            "workflow_id": context.workflow_id,
+            "reasoner_id": reasoner_name,
+            "agent_node_id": self.agent.node_id,
+        }
+
+        if parent_context and parent_context.execution_id:
+            payload["parent_execution_id"] = parent_context.execution_id
+
+        if context.session_id:
+            payload["session_id"] = context.session_id
+
+        try:
+            response = await async_request(
+                "POST",
+                f"{self.agent.brain_server}/api/v1/workflow/executions",
+                json=payload,
+                headers={"Content-Type": "application/json"},
+                timeout=2.0,
+            )
+        except Exception as exc:  # pragma: no cover - network failure
+            log_warn(f"Registration request failed for execution {context.execution_id}: {exc}")
+            raise
+
+        status = getattr(response, "status", None)
+        if status is None:
+            status = getattr(response, "status_code", None)
+
+        if status and status >= 400:
+            message = f"Registration failed with status {status}"
+            log_error(message)
+            raise RuntimeError(message)
+
+        registration_data = None
+        if hasattr(response, "json"):
+            try:
+                registration_data = response.json()
+            except Exception:  # pragma: no cover - best effort
+                registration_data = None
+
+        if isinstance(registration_data, dict):
+            if registration_data.get("execution_id"):
+                context.execution_id = registration_data["execution_id"]
+            if registration_data.get("workflow_id"):
+                context.workflow_id = registration_data["workflow_id"]
+            if registration_data.get("run_id"):
+                context.run_id = registration_data["run_id"]
+
+        context.registered = True
 
     async def execute_with_tracking(
         self, original_func: Callable, args: tuple, kwargs: dict
@@ -117,23 +189,31 @@ class AgentWorkflow:
             child_context = current_context.create_child_context()
             # 🔥 FIX: Update the child context's reasoner name with the actual function name
             child_context.reasoner_name = original_func.__name__
-            child_execution_id = child_context.execution_id  # Use the execution_id from the child context
+            await self._ensure_execution_registered(
+                child_context,
+                original_func.__name__,
+                current_context,
+            )
+            child_execution_id = child_context.execution_id
 
             if self.agent.dev_mode:
-                print(f"🔍 TRACK: Created child context for {original_func.__name__}")
-                print(f"  Parent Execution ID: {current_context.execution_id}")
-                print(f"  Child Execution ID: {child_execution_id}")
-                print(f"  Workflow ID: {child_context.workflow_id}")
-                print(f"  Parent Workflow ID: {child_context.parent_workflow_id}")
-                print(f"  Reasoner Name: {child_context.reasoner_name}")
+                log_debug(
+                    "TRACK: Created child context for "
+                    f"{original_func.__name__}\n"
+                    f"  Parent Execution ID: {current_context.execution_id}\n"
+                    f"  Child Execution ID: {child_execution_id}\n"
+                    f"  Workflow ID: {child_context.workflow_id}\n"
+                    f"  Parent Workflow ID: {child_context.parent_workflow_id}\n"
+                    f"  Reasoner Name: {child_context.reasoner_name}"
+                )
 
             # Inject execution context if function accepts it
             sig = inspect.signature(original_func)
             if "execution_context" in sig.parameters:
                 kwargs["execution_context"] = child_context
                 if self.agent.dev_mode:
-                    print(
-                        f"🔍 TRACK: Injected execution context into {original_func.__name__}"
+                    log_debug(
+                        f"TRACK: Injected execution context into {original_func.__name__}"
                     )
 
             # Set child context as current during execution
@@ -141,8 +221,8 @@ class AgentWorkflow:
             self.agent._current_execution_context = child_context
 
             if self.agent.dev_mode:
-                print(
-                    f"🔍 TRACK: Sending start notification for {original_func.__name__}"
+                log_debug(
+                    f"TRACK: Sending start notification for {original_func.__name__}"
                 )
 
             # 🔧 FIX: Capture original input data before function execution
@@ -169,21 +249,19 @@ class AgentWorkflow:
                     original_input_data[f"arg_{i}"] = arg
                 original_input_data.update(kwargs)
 
-            # Fire-and-forget start notification
-            asyncio.create_task(
-                self.notify_call_start(
-                    child_execution_id,
-                    child_context,
-                    original_func.__name__,
-                    original_input_data,  # 🔧 FIX: Send complete input data
-                    parent_execution_id=current_context.execution_id,
-                )
+            # Send start notification and wait for acknowledgement so ordering is preserved.
+            await self.notify_call_start(
+                child_execution_id,
+                child_context,
+                original_func.__name__,
+                original_input_data,  # 🔧 FIX: Send complete input data
+                parent_execution_id=current_context.execution_id,
             )
 
             start_time = time.time()
             try:
                 if self.agent.dev_mode:
-                    print(f"🔍 TRACK: Executing {original_func.__name__}")
+                    log_debug(f"TRACK: Executing {original_func.__name__}")
 
                 # 🔥 NEW: Automatic Pydantic model conversion (FastAPI-like behavior)
                 try:
@@ -192,7 +270,9 @@ class AgentWorkflow:
                         args = converted_args
                         kwargs = converted_kwargs
                         if self.agent.dev_mode:
-                            print(f"🔍 TRACK: Converted arguments for {original_func.__name__}")
+                            log_debug(
+                                f"TRACK: Converted arguments for {original_func.__name__}"
+                            )
                 except ValidationError as e:
                     # Re-raise validation errors with context
                     raise ValidationError(
@@ -202,7 +282,9 @@ class AgentWorkflow:
                 except Exception as e:
                     # Log conversion errors but continue with original args for backward compatibility
                     if self.agent.dev_mode:
-                        print(f"⚠️ Warning: Failed to convert arguments for {original_func.__name__}: {e}")
+                        log_warn(
+                            f"TRACK: Failed to convert arguments for {original_func.__name__}: {e}"
+                        )
 
                 # Execute the original function
                 if asyncio.iscoroutinefunction(original_func):
@@ -211,57 +293,69 @@ class AgentWorkflow:
                     result = original_func(*args, **kwargs)
 
                 if self.agent.dev_mode:
-                    print(
-                        f"🔍 TRACK: {original_func.__name__} completed successfully, sending completion notification"
+                    log_debug(
+                        f"TRACK: {original_func.__name__} completed successfully, sending completion notification"
                     )
 
-                # Fire-and-forget completion notification
                 end_time = time.time()
-                asyncio.create_task(
-                    self.notify_call_complete(
-                        child_execution_id,
-                        child_context.workflow_id,
-                        result,
-                        int((end_time - start_time) * 1000),
-                        child_context,  # 🔥 FIX: Pass child_context to get parent_workflow_id
-                        input_data=original_input_data,  # 🔧 FIX: Pass original complete input data
-                        parent_execution_id=current_context.execution_id,
-                    )
+                await self.notify_call_complete(
+                    child_execution_id,
+                    child_context.workflow_id,
+                    result,
+                    int((end_time - start_time) * 1000),
+                    child_context,  # 🔥 FIX: Pass child_context to get parent_workflow_id
+                    input_data=original_input_data,  # 🔧 FIX: Pass original complete input data
+                    parent_execution_id=current_context.execution_id,
                 )
 
                 return result
 
-            except Exception as e:
+            except asyncio.CancelledError as cancel_err:
                 if self.agent.dev_mode:
-                    print(
-                        f"🔍 TRACK: {original_func.__name__} failed with error: {e}, sending error notification"
+                    log_debug(
+                        f"TRACK: {original_func.__name__} cancelled, sending error notification"
                     )
 
-                # Fire-and-forget error notification
                 end_time = time.time()
-                asyncio.create_task(
-                    self.notify_call_error(
-                        child_execution_id,
-                        child_context.workflow_id,
-                        str(e),
-                        int((end_time - start_time) * 1000),
-                        child_context,  # 🔥 FIX: Pass child_context to get parent_workflow_id
-                        input_data=original_input_data,  # 🔧 FIX: Pass original complete input data
-                        parent_execution_id=current_context.execution_id,
+                await self.notify_call_error(
+                    child_execution_id,
+                    child_context.workflow_id,
+                    "Execution cancelled by upstream client",
+                    int((end_time - start_time) * 1000),
+                    child_context,
+                    input_data=original_input_data,
+                    parent_execution_id=current_context.execution_id,
+                )
+                raise cancel_err
+
+            except Exception as e:
+                if self.agent.dev_mode:
+                    log_error(
+                        f"TRACK: {original_func.__name__} failed with error: {e}, sending error notification"
                     )
+
+                end_time = time.time()
+                await self.notify_call_error(
+                    child_execution_id,
+                    child_context.workflow_id,
+                    str(e),
+                    int((end_time - start_time) * 1000),
+                    child_context,  # 🔥 FIX: Pass child_context to get parent_workflow_id
+                    input_data=original_input_data,  # 🔧 FIX: Pass original complete input data
+                    parent_execution_id=current_context.execution_id,
                 )
                 raise
             finally:
                 # Always restore previous context
                 self.agent._current_execution_context = previous_context
                 if self.agent.dev_mode:
-                    print(
-                        f"🔍 TRACK: Restored previous context for {original_func.__name__}"
+                    log_debug(
+                        f"TRACK: Restored previous context for {original_func.__name__}"
                     )
         else:
             if self.agent.dev_mode:
-                print(
-                    f"🔍 TRACK: No execution context found for {original_func.__name__}, creating root context"
+                log_debug(
+                    f"TRACK: No execution context found for {original_func.__name__}, creating root context"
                 )
 
             # Create a new root execution context for tracking
@@ -272,8 +366,10 @@ class AgentWorkflow:
             root_execution_id = root_context.execution_id  # Use the execution_id from the root context
 
             if self.agent.dev_mode:
-                print(
-                    f"🔍 TRACK: Created root context - workflow_id={root_context.workflow_id}, execution_id={root_execution_id}"
+                log_debug(
+                    "TRACK: Created root context\n"
+                    f"  Workflow ID: {root_context.workflow_id}\n"
+                    f"  Execution ID: {root_execution_id}"
                 )
 
             # Inject execution context if function accepts it
@@ -281,8 +377,8 @@ class AgentWorkflow:
             if "execution_context" in sig.parameters:
                 kwargs["execution_context"] = root_context
                 if self.agent.dev_mode:
-                    print(
-                        f"🔍 TRACK: Injected execution context into {original_func.__name__}"
+                    log_debug(
+                        f"TRACK: Injected execution context into {original_func.__name__}"
                     )
 
             # Set root context as current during execution
@@ -290,25 +386,23 @@ class AgentWorkflow:
             self.agent._current_execution_context = root_context
 
             if self.agent.dev_mode:
-                print(
-                    f"🔍 TRACK: Sending start notification for {original_func.__name__}"
+                log_debug(
+                    f"TRACK: Sending start notification for {original_func.__name__}"
                 )
 
-            # Fire-and-forget start notification
-            asyncio.create_task(
-                self.notify_call_start(
-                    root_execution_id,
-                    root_context,
-                    original_func.__name__,
-                    kwargs,
-                    parent_execution_id=None,  # This is a root context
-                )
+            # Ensure the root start notification lands before execution proceeds.
+            await self.notify_call_start(
+                root_execution_id,
+                root_context,
+                original_func.__name__,
+                kwargs,
+                parent_execution_id=None,  # This is a root context
             )
 
             start_time = time.time()
             try:
                 if self.agent.dev_mode:
-                    print(f"🔍 TRACK: Executing {original_func.__name__}")
+                    log_debug(f"TRACK: Executing {original_func.__name__}")
 
                 # 🔥 NEW: Automatic Pydantic model conversion (FastAPI-like behavior)
                 try:
@@ -317,7 +411,9 @@ class AgentWorkflow:
                         args = converted_args
                         kwargs = converted_kwargs
                         if self.agent.dev_mode:
-                            print(f"🔍 TRACK: Converted arguments for {original_func.__name__}")
+                            log_debug(
+                                f"TRACK: Converted arguments for {original_func.__name__}"
+                            )
                 except ValidationError as e:
                     # Re-raise validation errors with context
                     raise ValidationError(
@@ -327,7 +423,9 @@ class AgentWorkflow:
                 except Exception as e:
                     # Log conversion errors but continue with original args for backward compatibility
                     if self.agent.dev_mode:
-                        print(f"⚠️ Warning: Failed to convert arguments for {original_func.__name__}: {e}")
+                        log_warn(
+                            f"TRACK: Failed to convert arguments for {original_func.__name__}: {e}"
+                        )
 
                 # Execute the original function
                 if asyncio.iscoroutinefunction(original_func):
@@ -336,50 +434,61 @@ class AgentWorkflow:
                     result = original_func(*args, **kwargs)
 
                 if self.agent.dev_mode:
-                    print(
-                        f"🔍 TRACK: {original_func.__name__} completed successfully, sending completion notification"
+                    log_debug(
+                        f"TRACK: {original_func.__name__} completed successfully, sending completion notification"
                     )
 
-                # Fire-and-forget completion notification
                 end_time = time.time()
-                asyncio.create_task(
-                    self.notify_call_complete(
-                        root_execution_id,
-                        root_context.workflow_id,
-                        result,
-                        int((end_time - start_time) * 1000),
-                        root_context,  # 🔥 FIX: Pass root_context (parent_workflow_id will be None for root)
-                        input_data=kwargs,  # 🔧 FIX: Pass actual input data
-                    )
+                await self.notify_call_complete(
+                    root_execution_id,
+                    root_context.workflow_id,
+                    result,
+                    int((end_time - start_time) * 1000),
+                    root_context,  # 🔥 FIX: Pass root_context (parent_workflow_id will be None for root)
+                    input_data=kwargs,  # 🔧 FIX: Pass actual input data
                 )
 
                 return result
 
-            except Exception as e:
+            except asyncio.CancelledError as cancel_err:
                 if self.agent.dev_mode:
-                    print(
-                        f"🔍 TRACK: {original_func.__name__} failed with error: {e}, sending error notification"
+                    log_debug(
+                        f"TRACK: {original_func.__name__} cancelled, sending error notification"
                     )
 
-                # Fire-and-forget error notification
                 end_time = time.time()
-                asyncio.create_task(
-                    self.notify_call_error(
-                        root_execution_id,
-                        root_context.workflow_id,
-                        str(e),
-                        int((end_time - start_time) * 1000),
-                        root_context,  # 🔥 FIX: Pass root_context (parent_workflow_id will be None for root)
-                        input_data=kwargs,  # 🔧 FIX: Pass actual input data
+                await self.notify_call_error(
+                    root_execution_id,
+                    root_context.workflow_id,
+                    "Execution cancelled by upstream client",
+                    int((end_time - start_time) * 1000),
+                    root_context,
+                    input_data=kwargs,
+                )
+                raise cancel_err
+
+            except Exception as e:
+                if self.agent.dev_mode:
+                    log_error(
+                        f"TRACK: {original_func.__name__} failed with error: {e}, sending error notification"
                     )
+
+                end_time = time.time()
+                await self.notify_call_error(
+                    root_execution_id,
+                    root_context.workflow_id,
+                    str(e),
+                    int((end_time - start_time) * 1000),
+                    root_context,  # 🔥 FIX: Pass root_context (parent_workflow_id will be None for root)
+                    input_data=kwargs,  # 🔧 FIX: Pass actual input data
                 )
                 raise
             finally:
                 # Always restore previous context (which was None)
                 self.agent._current_execution_context = previous_context
                 if self.agent.dev_mode:
-                    print(
-                        f"🔍 TRACK: Restored previous context for {original_func.__name__}"
+                    log_debug(
+                        f"TRACK: Restored previous context for {original_func.__name__}"
                     )
 
     async def notify_call_start(
@@ -403,6 +512,7 @@ class AgentWorkflow:
                 "input_data": AgentUtils.serialize_result(input_data),
                 "started_at": time.time(),
                 "type": reasoner_name,
+                "run_id": context.run_id,
             }
 
             # Validation logging for parent-child relationships
@@ -458,12 +568,13 @@ class AgentWorkflow:
                 "parent_workflow_id": parent_workflow_id,  # 🔥 FIX: Now correctly includes parent_workflow_id
                 "agent_node_id": self.agent.node_id,
                 "reasoner_id": reasoner_name,  # 🔥 FIX: Use properly updated reasoner name
-                "status": "completed",
+                "status": "succeeded",
                 "input_data": serialized_input,  # 🔧 FIX: Send actual input data
                 "result": serialized_result,  # ✅ JSON-serializable
                 "duration_ms": duration_ms,
                 "completed_at": time.time(),
                 "type": reasoner_name,  # 🔥 FIX: Use properly updated reasoner name
+                "run_id": context.run_id,
             }
 
             # Validation logging for completion tracking
@@ -517,6 +628,7 @@ class AgentWorkflow:
                 "duration_ms": duration_ms,
                 "completed_at": time.time(),
                 "type": reasoner_name,  # 🔥 FIX: Use properly updated reasoner name
+                "run_id": context.run_id,
             }
 
             # Validation logging for error tracking
@@ -552,29 +664,26 @@ class AgentWorkflow:
             else:
                 log_fire("🔍 INPUT_DEBUG: Sending None input_data")
 
-            if aiohttp:
-                # Use aiohttp for non-blocking HTTP calls
-                timeout = aiohttp.ClientTimeout(total=1.0)  # 1 second timeout
-                async with aiohttp.ClientSession(timeout=timeout) as session:
-                    async with session.post(
-                        f"{self.agent.brain_server}/api/v1/workflow/update",
-                        json=payload,
-                        headers={"Content-Type": "application/json"},
-                    ) as response:
-                        log_fire(f"Response status: {response.status}")
-                        # Don't wait for response, just fire and forget
-                        pass
-            else:
-                # Fallback to httpx if aiohttp not available
-                import httpx
+            client = getattr(self.agent, "client", None)
+            async_request = getattr(client, "_async_request", None)
 
-                async with httpx.AsyncClient(timeout=1.0) as client:
-                    response = await client.post(
-                        f"{self.agent.brain_server}/api/v1/workflow/update",
-                        json=payload,
-                        headers={"Content-Type": "application/json"},
-                    )
-                    log_fire(f"Response status: {response.status_code}")
+            if async_request is None or not callable(async_request):
+                log_fire("Skipping workflow update - async request handler unavailable")
+                return
+
+            response = await async_request(
+                "POST",
+                f"{self.agent.brain_server}/api/v1/workflow/update",
+                json=payload,
+                headers={"Content-Type": "application/json"},
+                timeout=1.5,
+            )
+
+            status = getattr(response, "status", None)
+            if status is None:
+                status = getattr(response, "status_code", None)
+            if status is not None:
+                log_fire(f"Response status: {status}")
         except Exception as e:
             log_fire(f"Error sending workflow update: {e}")
             # Continue silently for fire-and-forget
