@@ -15,8 +15,11 @@ from typing import Any, Dict, List, Optional, Set, Tuple
 from urllib.parse import urljoin
 import uuid
 
+import aiohttp
+
 from .async_config import AsyncConfig
 from .execution_state import ExecutionState, ExecutionStatus, ExecutionPriority
+from .status import normalize_status
 from .http_connection_manager import ConnectionManager
 from .result_cache import ResultCache
 from .logger import get_logger
@@ -143,10 +146,15 @@ class AsyncExecutionManager:
         # Initialize components
         self.connection_manager = connection_manager or ConnectionManager(self.config)
         self.result_cache = result_cache or ResultCache(self.config)
-        
+
         # Execution tracking
         self._executions: Dict[str, ExecutionState] = {}
         self._execution_lock = asyncio.Lock()
+        capacity = max(1, self.config.max_concurrent_executions)
+        self._capacity_semaphore = asyncio.Semaphore(capacity)
+
+        # Event stream configuration
+        self._event_stream_headers: Dict[str, str] = {}
         
         # Polling coordination
         self._polling_task: Optional[asyncio.Task] = None
@@ -159,14 +167,28 @@ class AsyncExecutionManager:
         # Background tasks
         self._cleanup_task: Optional[asyncio.Task] = None
         self._metrics_task: Optional[asyncio.Task] = None
+        self._event_stream_task: Optional[asyncio.Task] = None
         
         # Circuit breaker state
         self._circuit_breaker_failures = 0
         self._circuit_breaker_last_failure = 0.0
         self._circuit_breaker_open = False
-        
+
         logger.debug(f"AsyncExecutionManager initialized with base_url={base_url}")
-    
+
+    def set_event_stream_headers(self, headers: Optional[Dict[str, str]]) -> None:
+        """Configure headers forwarded to the SSE event stream."""
+
+        if headers is None:
+            self._event_stream_headers = {}
+            return
+
+        self._event_stream_headers = {
+            key: value
+            for key, value in headers.items()
+            if value is not None
+        }
+
     async def __aenter__(self):
         """Async context manager entry."""
         await self.start()
@@ -185,18 +207,23 @@ class AsyncExecutionManager:
         """
         if self._polling_task is not None:
             raise RuntimeError("AsyncExecutionManager is already started")
-        
+
         # Start components
         await self.connection_manager.start()
         await self.result_cache.start()
-        
+
+        self._shutdown_event.clear()
+
         # Start background tasks
         self._polling_task = asyncio.create_task(self._polling_loop())
         self._cleanup_task = asyncio.create_task(self._cleanup_loop())
-        
+
         if self.config.enable_performance_logging:
             self._metrics_task = asyncio.create_task(self._metrics_loop())
-        
+
+        if self.config.enable_event_stream:
+            self._event_stream_task = asyncio.create_task(self._event_stream_loop())
+
         logger.info(f"AsyncExecutionManager started with max_concurrent={self.config.max_concurrent_executions}")
     
     async def stop(self) -> None:
@@ -212,7 +239,8 @@ class AsyncExecutionManager:
         tasks_to_cancel = [
             self._polling_task,
             self._cleanup_task,
-            self._metrics_task
+            self._metrics_task,
+            self._event_stream_task
         ]
         
         for task in tasks_to_cancel:
@@ -222,12 +250,18 @@ class AsyncExecutionManager:
                     await task
                 except asyncio.CancelledError:
                     pass
+
+        self._polling_task = None
+        self._cleanup_task = None
+        self._metrics_task = None
+        self._event_stream_task = None
         
         # Cancel all active executions
         async with self._execution_lock:
             for execution in self._executions.values():
                 if execution.is_active:
                     execution.cancel("Manager shutdown")
+                    self._release_capacity_for_execution(execution)
         
         # Stop components
         await self.connection_manager.close()
@@ -263,21 +297,22 @@ class AsyncExecutionManager:
         if self._polling_task is None:
             raise RuntimeError("AsyncExecutionManager is not started")
         
-        # Check capacity
-        async with self._execution_lock:
-            if len(self._executions) >= self.config.max_concurrent_executions:
-                raise RuntimeError(f"Maximum concurrent executions reached: {self.config.max_concurrent_executions}")
-        
         # Check circuit breaker
         if self._is_circuit_breaker_open():
             raise RuntimeError("Circuit breaker is open - too many recent failures")
-        
+
+        # Reserve capacity slot; released once terminal
+        await self._capacity_semaphore.acquire()
+
         # Prepare request
         url = urljoin(self.base_url, f"/api/v1/execute/async/{target}")
         request_headers = {
             'Content-Type': 'application/json',
             **(headers or {})
         }
+        payload = {
+            'input': input_data,
+        }
         
         # Set timeout
         execution_timeout = timeout or self.config.default_execution_timeout
@@ -288,7 +323,7 @@ class AsyncExecutionManager:
             async with self.connection_manager.get_session() as session:
                 response = await session.post(
                     url,
-                    json=input_data,
+                    json=payload,
                     headers=request_headers,
                     timeout=self.config.polling_timeout
                 )
@@ -327,6 +362,7 @@ class AsyncExecutionManager:
             return execution_id
             
         except Exception as e:
+            self._capacity_semaphore.release()
             self._record_circuit_breaker_failure()
             logger.error(f"Failed to submit execution for target {target}: {e}")
             raise
@@ -473,7 +509,7 @@ class AsyncExecutionManager:
     async def cleanup_completed_executions(self) -> int:
         """
         Clean up completed executions to manage memory.
-        
+
         Returns:
             int: Number of executions cleaned up
         """
@@ -481,40 +517,186 @@ class AsyncExecutionManager:
         current_time = time.time()
         
         async with self._execution_lock:
-            # Get completed executions sorted by completion time
-            completed_executions = [
-                (exec_id, execution) for exec_id, execution in self._executions.items()
+            # Collect terminal executions for retention analysis
+            completed_executions = {
+                exec_id: execution
+                for exec_id, execution in self._executions.items()
                 if execution.is_terminal
+            }
+
+            if not completed_executions:
+                return 0
+
+            removal_candidates = set()
+
+            # Time-based pruning to keep memory bounded during long-running sessions
+            retention_seconds = self.config.completed_execution_retention_seconds
+            if retention_seconds > 0:
+                for exec_id, execution in completed_executions.items():
+                    end_time = execution.metrics.end_time or execution.metrics.submit_time
+                    if end_time and (current_time - end_time) > retention_seconds:
+                        removal_candidates.add(exec_id)
+
+            # Enforce cap on stored completions after time-based pruning
+            remaining = [
+                (exec_id, execution)
+                for exec_id, execution in completed_executions.items()
+                if exec_id not in removal_candidates
             ]
-            
-            # Keep only the most recent completed executions
-            if len(completed_executions) > self.config.max_completed_executions:
-                # Sort by end time (oldest first)
-                completed_executions.sort(key=lambda x: x[1].metrics.end_time or 0)
-                
-                # Remove oldest executions
-                to_remove = len(completed_executions) - self.config.max_completed_executions
-                for i in range(to_remove):
-                    exec_id, execution = completed_executions[i]
-                    
-                    # Cache successful results before removal
-                    if execution.is_successful and execution.result is not None:
-                        self.result_cache.set_execution_result(exec_id, execution.result)
-                    
-                    del self._executions[exec_id]
-                    cleanup_count += 1
+
+            if len(remaining) > self.config.max_completed_executions:
+                # Remove the oldest executions first
+                remaining.sort(key=lambda item: item[1].metrics.end_time or 0)
+                overflow = len(remaining) - self.config.max_completed_executions
+                for i in range(overflow):
+                    removal_candidates.add(remaining[i][0])
+
+            # Apply removals and cache results where applicable
+            for exec_id in removal_candidates:
+                execution = completed_executions.get(exec_id)
+                if execution is None:
+                    continue
+
+                if execution.is_successful and execution.result is not None:
+                    self.result_cache.set_execution_result(exec_id, execution.result)
+
+                self._release_capacity_for_execution(execution)
+                self._executions.pop(exec_id, None)
+                cleanup_count += 1
         
         if cleanup_count > 0:
             self.metrics.cleanup_operations += 1
             self.metrics.last_cleanup = current_time
             logger.debug(f"Cleaned up {cleanup_count} completed executions")
-        
+
         return cleanup_count
-    
+
+    async def _event_stream_loop(self) -> None:
+        """Listen for execution events over SSE and nudge polling."""
+        logger.debug("Starting event stream loop")
+
+        url = urljoin(self.base_url, self.config.event_stream_path)
+        backoff = max(self.config.event_stream_retry_backoff, 0.5)
+
+        while not self._shutdown_event.is_set():
+            try:
+                request_headers = {"Accept": "text/event-stream"}
+                if self._event_stream_headers:
+                    request_headers.update(self._event_stream_headers)
+
+                async with self.connection_manager.get_session() as session:
+                    timeout = aiohttp.ClientTimeout(total=None, sock_read=None)
+                    async with session.get(url, headers=request_headers, timeout=timeout) as response:
+                        if response.status != 200:
+                            body = await response.text()
+                            logger.warn(
+                                f"Event stream returned {response.status} for {url}: {body[:256]}"
+                            )
+                            await asyncio.sleep(backoff)
+                            continue
+
+                        buffer = ""
+                        async for chunk in response.content.iter_chunked(1024):
+                            if self._shutdown_event.is_set():
+                                break
+                            if not chunk:
+                                continue
+                            try:
+                                decoded = chunk.decode("utf-8", errors="ignore")
+                            except Exception:
+                                continue
+
+                            buffer += decoded
+
+                            while "\n\n" in buffer:
+                                raw_event, buffer = buffer.split("\n\n", 1)
+                                data_lines = []
+                                for line in raw_event.splitlines():
+                                    if line.startswith(":"):
+                                        continue
+                                    if line.startswith("data:"):
+                                        data_lines.append(line[5:].lstrip())
+
+                                if not data_lines:
+                                    continue
+
+                                payload_str = "\n".join(data_lines).strip()
+                                if not payload_str:
+                                    continue
+
+                                try:
+                                    payload = json.loads(payload_str)
+                                except json.JSONDecodeError:
+                                    logger.debug(f"Failed to decode SSE payload: {payload_str[:120]}")
+                                    continue
+
+                                await self._handle_event_stream_payload(payload)
+
+            except asyncio.CancelledError:
+                break
+            except Exception as e:
+                if self._shutdown_event.is_set():
+                    break
+                logger.warn(f"Event stream error: {e}")
+                await asyncio.sleep(backoff)
+
+        logger.debug("Event stream loop stopped")
+
+    async def _handle_event_stream_payload(self, payload: Dict[str, Any]) -> None:
+        """Process a single SSE payload."""
+        execution_id = payload.get("execution_id") or payload.get("executionId")
+        if not execution_id:
+            return
+
+        schedule_poll = False
+        status_hint = normalize_status(payload.get("status"))
+        event_type = str(payload.get("type", "")).lower()
+
+        async with self._execution_lock:
+            execution = self._executions.get(execution_id)
+            if execution is None:
+                return
+
+            if event_type == "execution_started" or status_hint == "running":
+                execution.update_status(ExecutionStatus.RUNNING)
+            elif status_hint == "queued":
+                execution.update_status(ExecutionStatus.QUEUED)
+            elif status_hint == "pending":
+                execution.update_status(ExecutionStatus.PENDING)
+            elif status_hint in {"succeeded", "failed", "cancelled", "timeout"} or event_type in {"execution_completed", "execution_failed"}:
+                if status_hint == "failed":
+                    execution.update_status(ExecutionStatus.FAILED)
+                elif status_hint == "cancelled":
+                    execution.update_status(ExecutionStatus.CANCELLED)
+                elif status_hint == "timeout":
+                    execution.update_status(ExecutionStatus.TIMEOUT)
+                else:
+                    execution.update_status(ExecutionStatus.SUCCEEDED)
+                schedule_poll = True
+
+        if schedule_poll:
+            asyncio.create_task(self._poll_execution_immediate(execution_id))
+
+    async def _poll_execution_immediate(self, execution_id: str) -> None:
+        """Trigger an immediate poll for the provided execution."""
+        async with self._execution_lock:
+            execution = self._executions.get(execution_id)
+
+        if execution is None:
+            return
+
+        if execution.is_terminal and execution.result is not None:
+            return
+
+        try:
+            await self._poll_single_execution(execution)
+        except Exception as exc:
+            logger.debug(f"Immediate poll for {execution_id[:8]}... failed: {exc}")
+
     async def start_polling_task(self) -> None:
         """
         Start the background polling task.
-        
+
         Note: This is automatically called by start() and should not be called manually.
         """
         if self._polling_task is None or self._polling_task.done():
@@ -643,10 +825,9 @@ class AsyncExecutionManager:
             # Create batch requests
             requests = []
             for execution in batch:
-                url = urljoin(self.base_url, f"/api/v1/executions/{execution.execution_id}/status")
                 requests.append({
                     'method': 'GET',
-                    'url': url,
+                    'url': self._execution_status_url(execution.execution_id),
                     'timeout': self.config.polling_timeout
                 })
             
@@ -680,8 +861,8 @@ class AsyncExecutionManager:
     
     async def _poll_single_execution(self, execution: ExecutionState) -> None:
         """Poll a single execution for status updates."""
-        url = urljoin(self.base_url, f"/api/v1/executions/{execution.execution_id}/status")
-        
+        url = self._execution_status_url(execution.execution_id)
+
         start_time = time.time()
         try:
             response = await self.connection_manager.request(
@@ -746,6 +927,10 @@ class AsyncExecutionManager:
         finally:
             # Record metrics
             self.metrics.polling_metrics.record_poll(success, duration, timeout_occurred)
+
+    def _execution_status_url(self, execution_id: str) -> str:
+        """Return the canonical status endpoint for an execution."""
+        return urljoin(self.base_url, f"/api/v1/executions/{execution_id}")
     
     async def _update_execution_from_status(
         self,
@@ -753,34 +938,29 @@ class AsyncExecutionManager:
         status_data: Dict[str, Any]
     ) -> None:
         """Update execution state from status response."""
-        status_str = status_data.get('status', '').lower()
-        
-        # Map status string to enum
-        status_mapping = {
-            'queued': ExecutionStatus.QUEUED,
-            'running': ExecutionStatus.RUNNING,
-            'completed': ExecutionStatus.COMPLETED,
-            'failed': ExecutionStatus.FAILED,
-            'cancelled': ExecutionStatus.CANCELLED,
-            'timeout': ExecutionStatus.TIMEOUT
-        }
-        
-        new_status = status_mapping.get(status_str)
-        if new_status is None:
-            logger.warn(f"Unknown status '{status_str}' for execution {execution.execution_id[:8]}...")
+        raw_status = status_data.get("status")
+        normalized = normalize_status(raw_status)
+
+        try:
+            new_status = ExecutionStatus(normalized)
+        except ValueError:
+            logger.warning(
+                "Unknown status '%s' for execution %s", normalized, execution.execution_id[:8]
+            )
             return
-        
+
         old_status = execution.status
-        
+
         # Update status
         if new_status != old_status:
-            if new_status == ExecutionStatus.COMPLETED:
+            if new_status == ExecutionStatus.SUCCEEDED:
                 result = status_data.get('result')
                 execution.set_result(result)
                 
                 async with self._execution_lock:
                     self.metrics.completed_executions += 1
                     self.metrics.active_executions -= 1
+                self._release_capacity_for_execution(execution)
                 
             elif new_status == ExecutionStatus.FAILED:
                 error_msg = status_data.get('error', 'Execution failed')
@@ -790,12 +970,42 @@ class AsyncExecutionManager:
                 async with self._execution_lock:
                     self.metrics.failed_executions += 1
                     self.metrics.active_executions -= 1
-                
+                self._release_capacity_for_execution(execution)
+            elif new_status == ExecutionStatus.CANCELLED:
+                execution.update_status(new_status)
+
+                async with self._execution_lock:
+                    self.metrics.cancelled_executions += 1
+                    self.metrics.active_executions -= 1
+                self._release_capacity_for_execution(execution)
+
+            elif new_status == ExecutionStatus.TIMEOUT:
+                execution.update_status(new_status)
+
+                async with self._execution_lock:
+                    self.metrics.timeout_executions += 1
+                    self.metrics.active_executions -= 1
+                self._release_capacity_for_execution(execution)
+
             else:
                 execution.update_status(new_status)
-            
-            logger.debug(f"Execution {execution.execution_id[:8]}... status: {old_status.value} -> {new_status.value}")
-    
+
+            old_repr = getattr(old_status, "value", old_status)
+            new_repr = getattr(new_status, "value", new_status)
+            logger.debug(
+                f"Execution {execution.execution_id[:8]}... status: {old_repr} -> {new_repr}"
+            )
+
+    def _release_capacity_for_execution(self, execution: ExecutionState) -> None:
+        if getattr(execution, "_capacity_released", False):
+            return
+        execution._capacity_released = True
+        try:
+            self._capacity_semaphore.release()
+        except ValueError:
+            # Semaphore already fully released (can occur during shutdown cleanup)
+            pass
+
     async def _cleanup_loop(self) -> None:
         """Background task for periodic cleanup of completed executions."""
         logger.debug("Starting cleanup loop")
@@ -870,4 +1080,4 @@ class AsyncExecutionManager:
             f"total_executions={self.metrics.total_executions}, "
             f"success_rate={self.metrics.success_rate:.1f}%"
             f")"
-        )
\ No newline at end of file
+        )
