@@ -2,7 +2,6 @@ import asyncio
 import json
 import subprocess
 import uuid
-import logging
 from typing import Dict, List, Any, Optional, Tuple, Union
 from dataclasses import dataclass
 from fastapi import FastAPI, HTTPException
@@ -12,7 +11,9 @@ import signal
 import os
 from contextlib import asynccontextmanager
 
-logger = logging.getLogger(__name__)
+from .logger import get_logger
+
+logger = get_logger(__name__)
 
 @dataclass
 class PendingRequest:
@@ -62,7 +63,10 @@ class StdioMCPBridge:
         """Start the stdio MCP server and HTTP bridge"""
         try:
             if self.dev_mode:
-                logger.info(f"Starting stdio MCP bridge for {self.server_config.get('alias', 'unknown')} on port {self.port}")
+                logger.debug(
+                    f"Starting stdio MCP bridge for {self.server_config.get('alias', 'unknown')} "
+                    f"on port {self.port}"
+                )
             
             # Start the stdio MCP server process
             if not await self._start_stdio_process():
@@ -99,7 +103,7 @@ class StdioMCPBridge:
             self.server_task = asyncio.create_task(server.serve())
             
             if self.dev_mode:
-                logger.info(f"Stdio MCP bridge started successfully on port {self.port}")
+                logger.debug(f"Stdio MCP bridge started successfully on port {self.port}")
             
             return True
             
@@ -111,7 +115,7 @@ class StdioMCPBridge:
     async def stop(self) -> None:
         """Stop the bridge and cleanup resources"""
         if self.dev_mode:
-            logger.info("Stopping stdio MCP bridge...")
+            logger.debug("Stopping stdio MCP bridge...")
         
         self.running = False
         
@@ -155,7 +159,7 @@ class StdioMCPBridge:
                 logger.error(f"Error stopping process: {e}")
         
         if self.dev_mode:
-            logger.info("Stdio MCP bridge stopped")
+            logger.debug("Stdio MCP bridge stopped")
     
     async def _wait_for_process(self):
         """Wait for process to terminate"""
@@ -190,8 +194,8 @@ class StdioMCPBridge:
             env.update(self.server_config.get("environment", {}))
             
             if self.dev_mode:
-                logger.info(f"Starting process: {run_command}")
-                logger.info(f"Working directory: {working_dir}")
+                logger.debug(f"Starting process: {run_command}")
+                logger.debug(f"Working directory: {working_dir}")
             
             # Start process
             self.process = await asyncio.create_subprocess_shell(
@@ -235,7 +239,7 @@ class StdioMCPBridge:
         """Initialize MCP session with handshake"""
         try:
             if self.dev_mode:
-                logger.info("Initializing MCP session...")
+                logger.debug("Initializing MCP session...")
             
             # Send initialize request
             init_params = {
@@ -260,7 +264,7 @@ class StdioMCPBridge:
             self.initialized = True
             
             if self.dev_mode:
-                logger.info("MCP session initialized successfully")
+                logger.debug("MCP session initialized successfully")
             
             return True
             
@@ -536,4 +540,4 @@ class StdioMCPBridge:
             self.pending_requests.pop(request_id, None)
             
         if expired_ids and self.dev_mode:
-            logger.warning(f"Cleaned up {len(expired_ids)} expired requests")
\ No newline at end of file
+            logger.warning(f"Cleaned up {len(expired_ids)} expired requests")
