@@ -204,6 +204,11 @@ class AIConfig(BaseModel):
     avg_chars_per_token: int = Field(default=4, description="Average characters per token for approximation")
     max_input_tokens: Optional[int] = Field(default=None, description="Maximum input context tokens (overrides auto-detection)")
 
+    # Pydantic V2: allow fields that start with `model_`
+    model_config = {
+        "protected_namespaces": ()
+    }
+
     # Fallback model context mappings for when LiteLLM detection fails
     _MODEL_CONTEXT_LIMITS = {
         # OpenRouter Gemini models
@@ -395,7 +400,7 @@ class AIConfig(BaseModel):
 
     def to_dict(self) -> Dict[str, Any]:
         """Convert to dictionary representation"""
-        return self.dict()
+        return self.model_dump()
 
     @classmethod
     def from_env(cls, **overrides) -> "AIConfig":
